

#if !defined(LIR_CHAIN_WRITER_H)
#define LIR_CHAIN_WRITER_H

namespace nanojit
{
class LirOptimizationPass;

// allow easy manipulation of LIR instructions
class LirChainWriter
{
private:
	LirStorage                      _storage;
	Allocator*                      _allocator;

#if defined(NANOJIT_DEBUG)
	int                             m_iWasPeeked;
#endif

public:
									LirChainWriter();
									~LirChainWriter();

	void                            initialize(Allocator* allocator);
	void                            shutdown();

	void                            start();
	void                            end();
	void                            clear();

	LirStorage*                     getStorage();
	nanojit::LIns*                  getLastIns();

	Allocator*                      getAllocator();

public:
	LIns*                       ins0(LOpcode v);
	LIns*                       ins1(LOpcode v, LIns* a);
	LIns*                       ins2(LOpcode v, LIns* a, LIns* b);
	LIns*                       ins3(LOpcode v, LIns* a, LIns* b, LIns* c);
	LIns*                       insLabel();
	LIns*                       insPhi(LOpcode op, PhiData* phiData, LIns** values, size_t count, LIns* reconcileNodeNext);
	LIns*                       insSpillFence(LIns** valueArray, size_t count);
	LIns*                       insSpillFenceAll();
	LIns*                       insRegisterFence(LIns** valueArray, size_t count);
	LIns*                       insRegisterFenceAll();
	LIns*                       insKillFence(LIns** valueArray, size_t count);
	LIns*                       insBranch(LOpcode v, LIns* condition, LIns* to);
	LIns*                       insEntry(RegisterMask saveMask);
	LIns*                       insExit(ParameterValue* values, unsigned int count, RegisterMask restoreMask);
	LIns*                       insParam(LOpcode op, Register reg);
	LIns*                       insImmI(int32_t imm);
#if defined(NANOJIT_64BIT)
	LIns*                       insImmQ(int64_t imm);
#endif
	LIns*                       insImmF(float f);
	LIns*                       insImmF4(const float* f4);
	LIns*                       insImmD(double d);

	inline LIns* insImmP(const  void *ptr) {
#ifdef NANOJIT_64BIT
			return insImmQ((int64_t)ptr);
#else
			return insImmI((int32_t)ptr);
#endif
	}

	LIns*                       insLoad(LOpcode op, LIns* base, int32_t d, AccSet accSet, LoadQual loadQual);
	LIns*                       insStore(LOpcode op, LIns* value, LIns* base, int32_t d, AccSet accSet);
	LIns*                       insCall(LOpcode op, CallParameters* parameters, const void* address);
	LIns*                       insComment(const char* str);
	LIns*                       insSkip(LIns* skipTo);
	LIns*                       insSwz(LIns* a, uint8_t mask);

	LIns*                       insCopy(LIns* insToCopy);

	LIns*                       peekNext();

	void                        moveBefore(LIns* insToMove, LIns* insTarget);
	void                        moveAfter(LIns* insToMove, LIns* insTarget);

};

inline LirStorage* LirChainWriter::getStorage()
{
	return &_storage;
}

inline nanojit::LIns* LirChainWriter::getLastIns()
{
	return _storage.getReverseBegin().getElement<LInsElement>()->data();
}

inline Allocator* LirChainWriter::getAllocator()
{
	return _allocator;
}

inline LIns* LirChainWriter::peekNext()
{
#if defined(NANOJIT_DEBUG)
	m_iWasPeeked = 2;
#endif

	return _storage.peekNextElement()->data();
}


}

#endif // LIR_CHAIN_WRITER_H

