
#if !defined(__nanojit_LirInstPrinter__)
#define __nanojit_LirInstPrinter__


namespace nanojit
{
#if defined(NANOJIT_VERBOSE)
	class LInsPrinter
	{
	private:
		Allocator* alloc;
		const int EMB_NUM_USED_ACCS;

		char *formatImmI(RefBuf* buf, int32_t c);
#ifdef NANOJIT_64BIT
		char *formatImmQ(RefBuf* buf, uint64_t c);
#endif
		char *formatImmD(RefBuf* buf, double c);

	public:
		static const char* accNames[];                  // defined by the embedder

		LInsPrinter(Allocator* alloc, int embNumUsedAccs)
			: alloc(alloc), EMB_NUM_USED_ACCS(embNumUsedAccs), addrNameMap(NULL), lirNameMap(NULL)
		{
		}

		void initialize() { addrNameMap = new (alloc) AddrNameMap(alloc); lirNameMap = new (alloc) LirNameMap(alloc); }
		void shutdown() { addrNameMap  = NULL; lirNameMap = NULL; };

		char *formatAddr(RefBuf* buf, void* p);
		char *formatRef(RefBuf* buf, LIns* ref, bool showImmValue = true);
		char *formatIns(InsBuf* buf, LIns* ins);
		char *formatAccSet(RefBuf* buf, AccSet accSet);

		const char* lookupName(LIns* ins) { return lirNameMap->lookupName(ins); }
		void addName(LIns* ins, const char* name) { lirNameMap->addName(ins, name); }

	private:
		AddrNameMap*    addrNameMap;
		LirNameMap*     lirNameMap;
	};
#endif
}


#endif // __nanojit_LirInstPrinter__

